==> generic-plugin-3.6.1-sources.jar: com/xebialabs/deployit/plugin/generic/step/GenericBaseStep.java <==
package com.xebialabs.deployit.plugin.generic.step;

import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import com.google.common.collect.Iterables;
import com.google.common.io.ByteStreams;
import com.google.common.io.Closeables;
import com.google.common.io.Resources;
import com.xebialabs.deployit.plugin.api.deployment.execution.DeploymentExecutionContext;
import com.xebialabs.deployit.plugin.api.deployment.execution.DeploymentStep;
import com.xebialabs.deployit.plugin.generic.freemarker.CiAwareObjectWrapper;
import com.xebialabs.deployit.plugin.generic.freemarker.ConfigurationHolder;
import com.xebialabs.deployit.plugin.generic.freemarker.FileUploader;
import com.xebialabs.deployit.plugin.overthere.HostContainer;
import com.xebialabs.overthere.OperatingSystemFamily;
import com.xebialabs.overthere.OverthereConnection;
import com.xebialabs.overthere.OverthereFile;
import com.xebialabs.overthere.RuntimeIOException;
import com.xebialabs.overthere.local.LocalConnection;
import com.xebialabs.overthere.local.LocalFile;
import com.xebialabs.overthere.util.OverthereUtils;
import freemarker.ext.dom.NodeModel;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import java.io.*;
import java.net.URL;
import java.util.Map;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.base.Strings.emptyToNull;
import static com.google.common.collect.Maps.newHashMap;

@SuppressWarnings("serial")
public abstract class GenericBaseStep implements DeploymentStep {

    public static final String FREEMARKER_FILE_EXT = ".ftl";

    private int order;
    private String description;
    private HostContainer container;

    private String remoteWorkingDirPath;
    private boolean retainRemoteWorkingDirOnCompletion;

    private transient OverthereConnection localConn;
    private transient OverthereConnection remoteConn;
    private transient DeploymentExecutionContext ctx;
    private transient OverthereFile remoteWorkingDir;

    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    protected GenericBaseStep(int order, String description, HostContainer container) {
        this.order = order;
        this.description = description;
        this.container = container;
    }

    protected GenericBaseStep(int order, HostContainer container) {
        this(order, null, container);
    }

    @Override
    public int getOrder() {
        return order;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public Result execute(DeploymentExecutionContext ctx) throws Exception {
        try {
            this.ctx = ctx;
            return doExecute();
        } finally {
            disconnect();
        }
    }

    protected abstract Result doExecute() throws Exception;

    protected DeploymentExecutionContext getCtx() {
        return ctx;
    }

    protected HostContainer getContainer() {
        return container;
    }

    public OverthereFile getRemoteWorkingDirectory() {
        if (remoteWorkingDir == null) {
            OverthereFile tempDir;
            if (Strings.isNullOrEmpty(getRemoteWorkingDirPath())) {
                tempDir = getRemoteConnection().getTempFile("generic_plugin", ".tmp");
            } else {
                tempDir = getRemoteConnection().getFile(getRemoteWorkingDirPath());
            }
            tempDir.mkdir();
            remoteWorkingDir = tempDir;
        }
        return remoteWorkingDir;
    }


    public OverthereConnection getLocalConnection() {
        if (localConn == null) {
            localConn = LocalConnection.getLocalConnection();
        }
        return localConn;
    }

    public OverthereConnection getRemoteConnection() {
        if (remoteConn == null) {
            remoteConn = container.getHost().getConnection();
        }
        return remoteConn;
    }

    protected void disconnect() {
        if (localConn != null)
            localConn.close();

        if (!Strings.isNullOrEmpty(getRemoteWorkingDirPath()) && !isRetainRemoteWorkingDirOnCompletion()) {
            getRemoteWorkingDirectory().deleteRecursively();
        }

        if (remoteConn != null)
            remoteConn.close();

        remoteWorkingDir = null;
        localConn = null;
        remoteConn = null;
    }

    public String getRemoteWorkingDirPath() {
        return remoteWorkingDirPath;
    }

    public void setRemoteWorkingDirPath(String remoteWorkingDirPath) {
        this.remoteWorkingDirPath = remoteWorkingDirPath;
    }

    public boolean isRetainRemoteWorkingDirOnCompletion() {
        return retainRemoteWorkingDirOnCompletion;
    }

    public void setRetainRemoteWorkingDirOnCompletion(boolean deleteWorkingDirOnCompletion) {
        this.retainRemoteWorkingDirOnCompletion = deleteWorkingDirOnCompletion;
    }

    public OverthereFile uploadResourceToWorkingDirectory(String resource) {
        final String fileName = substringAfterLast(resource, '/', resource);
        return uploadToWorkingDirectory(Resources.getResource(resource), fileName);
    }


    public OverthereFile uploadToWorkingDirectory(String content, String fileName) {
        OverthereFile target = getRemoteWorkingDirectory().getFile(fileName);
        OverthereUtils.write(content.getBytes(), target);
        return target;
    }

    public OverthereFile uploadToWorkingDirectory(File content, String fileName) {
        OverthereFile target = getRemoteWorkingDirectory().getFile(fileName);
        LocalFile.valueOf(content).copyTo(target);
        return target;
    }

    public OverthereFile uploadToWorkingDirectory(URL content, String fileName) {
        OverthereFile target = getRemoteWorkingDirectory().getFile(fileName);
        try {
            Resources.copy(content, target.getOutputStream());
        } catch (IOException e) {
            throw new RuntimeIOException(e);
        }
        return target;
    }

    public boolean hostFileExists(String remoteFile) {
        checkNotNull(emptyToNull(remoteFile));
        OverthereFile file = getRemoteConnection().getFile(remoteFile);
        return file.exists();
    }

    public String getHostLineSeparator() {
        return getContainer().getHost().getOs().getLineSeparator();
    }

    public String getHostFileSeparator() {
        return getContainer().getHost().getOs().getFileSeparator();
    }

    public String readHostFile(final String remoteFile) {
        checkNotNull(emptyToNull(remoteFile));
        OverthereFile file = getRemoteConnection().getFile(remoteFile);
        checkArgument(file.exists(), "File %s does not exist on host %s", remoteFile, getContainer().getHost());
        try {
            byte[] bytes = ByteStreams.toByteArray(file.getInputStream());
            return new String(bytes);
        } catch (IOException e) {
            throw new RuntimeIOException("Failed to read file " + remoteFile, e);
        }
    }

    public String[] readHostFileLines(final String remoteFile) {
        String data = readHostFile(remoteFile);
        Iterable<String> iterable = Splitter.on(getHostLineSeparator()).split(data);
        return Iterables.toArray(iterable, String.class);
    }

    public void createOrReplaceHostFile(String content, String remoteFile) {
        checkNotNull(emptyToNull(remoteFile));
        checkNotNull(content);
        OverthereFile file = getRemoteConnection().getFile(remoteFile);
        if (file.exists()) {
            logger.debug("File " + remoteFile + " already exists. Will delete before attempting to write.");
            file.delete();
        }

	    final ByteArrayInputStream from = new ByteArrayInputStream(content.getBytes());
	    final OutputStream to = file.getOutputStream();
	    try {
		    ByteStreams.copy(from, to);
        } catch (IOException e) {
            throw new RuntimeIOException("Failed to write to " + remoteFile, e);
        }
	    finally {
		    Closeables.closeQuietly(from);
		    Closeables.closeQuietly(to);
	    }
    }

    public NodeModel readHostXmlFileAsModel(final String remoteXmlFile) {
        checkNotNull(emptyToNull(remoteXmlFile));
        OverthereFile file = getRemoteConnection().getFile(remoteXmlFile);
        checkArgument(file.exists(), "File %s does not exist on host %s", remoteXmlFile, getContainer().getHost());
        InputStream remoteXmlStream = file.getInputStream();
        try {
            return NodeModel.parse(new InputSource(remoteXmlStream), false, true);
        } catch (ParserConfigurationException exc) {
            throw new RuntimeException("Cannot read xml file " + remoteXmlFile, exc);
        } catch (SAXException exc) {
            throw new RuntimeException("Cannot read xml file " + remoteXmlFile, exc);
        } catch (IOException exc) {
            throw new RuntimeIOException("Cannot read xml file " + remoteXmlFile, exc);
        } finally {
            Closeables.closeQuietly(remoteXmlStream);
        }
    }

    public String evaluateTemplate(String templatePath, Map<String, Object> vars) {
        Configuration cfg = ConfigurationHolder.getConfiguration();
        try {
            Template template = cfg.getTemplate(templatePath);
            StringWriter sw = new StringWriter();
            template.createProcessingEnvironment(vars, sw, new CiAwareObjectWrapper(new WorkingFolderUploader())).process();
            return sw.toString();
        } catch (IOException e) {
            throw new RuntimeIOException(e);
        } catch (TemplateException e) {
            throw new RuntimeException(e);
        }
    }

    public String resolveOsSpecificTemplate(String template) {
        if (classpathResourceExists(template + FREEMARKER_FILE_EXT)) {
            return template + FREEMARKER_FILE_EXT;
        }

        String osSpecificTemplate = template;

        String scriptExt = substringAfterLast(osSpecificTemplate, '.');
        if (scriptExt == null) {
            OperatingSystemFamily os = getContainer().getHost().getOs();
            osSpecificTemplate = osSpecificTemplate + os.getScriptExtension();
        }

        if (!classpathResourceExists(osSpecificTemplate)) {
            String osSpecificScriptTemplate = osSpecificTemplate + FREEMARKER_FILE_EXT;
            if (!classpathResourceExists(osSpecificScriptTemplate))
                throw new IllegalArgumentException("Resource " + osSpecificTemplate + " not found in classpath");
            else
                osSpecificTemplate = osSpecificScriptTemplate;
        }

        return osSpecificTemplate;
    }

    public boolean classpathResourceExists(String resource) {
        return Thread.currentThread().getContextClassLoader().getResource(resource) != null;
    }

    protected void evaluateTemplate(OverthereFile renderTo, String templatePath, Map<String, Object> vars) {
        Configuration cfg = ConfigurationHolder.getConfiguration();
        OutputStream out = renderTo.getOutputStream();
        try {
            Template template = cfg.getTemplate(templatePath);
            template.process(vars, new OutputStreamWriter(out));

            if (logger.isDebugEnabled()) {
                InputStream in = renderTo.getInputStream();
                try {
                    logger.debug(new String(ByteStreams.toByteArray(in)));
                } finally {
                    Closeables.closeQuietly(in);
                }
            }

        } catch (IOException e) {
            throw new RuntimeIOException(e);
        } catch (TemplateException e) {
            throw new RuntimeException(e);
        } finally {
            Closeables.closeQuietly(out);
        }
    }

    public String substringAfterLast(String str, char sub, String defaultValue) {
        String s = substringAfterLast(str, sub);
        if (s == null) {
            return defaultValue;
        }
        return s;
    }

    public String substringAfterLast(String str, char sub) {
        int pos = str.lastIndexOf(sub);
        if (pos == -1) {
            return null;
        } else {
            return str.substring(pos + 1);
        }
    }

    private class WorkingFolderUploader implements FileUploader {
        private Map<String, String> uploadedFiles = newHashMap();

        @Override
        public String upload(OverthereFile file) {
            if (uploadedFiles.containsKey(file.getName())) {
                return uploadedFiles.get(file.getName());
            }
            OverthereFile uploadedFile = getRemoteWorkingDirectory().getFile(file.getName());
            if (!uploadedFile.exists()) {
                file.copyTo(uploadedFile);
            }
            uploadedFiles.put(file.getName(), uploadedFile.getPath());
            return uploadedFile.getPath();
        }
    }
}

==> generic-plugin-3.6.2-sources.jar: com/xebialabs/deployit/plugin/generic/step/GenericBaseStep.java <==
package com.xebialabs.deployit.plugin.generic.step;

import com.google.common.base.Splitter;
import com.google.common.base.Strings;
import com.google.common.collect.Iterables;
import com.google.common.io.ByteStreams;
import com.google.common.io.Closeables;
import com.google.common.io.Resources;
import com.xebialabs.deployit.plugin.api.deployment.execution.DeploymentExecutionContext;
import com.xebialabs.deployit.plugin.api.deployment.execution.DeploymentStep;
import com.xebialabs.deployit.plugin.generic.freemarker.CiAwareObjectWrapper;
import com.xebialabs.deployit.plugin.generic.freemarker.ConfigurationHolder;
import com.xebialabs.deployit.plugin.generic.freemarker.FileUploader;
import com.xebialabs.deployit.plugin.overthere.HostContainer;
import com.xebialabs.overthere.OperatingSystemFamily;
import com.xebialabs.overthere.OverthereConnection;
import com.xebialabs.overthere.OverthereFile;
import com.xebialabs.overthere.RuntimeIOException;
import com.xebialabs.overthere.local.LocalConnection;
import com.xebialabs.overthere.local.LocalFile;
import com.xebialabs.overthere.util.OverthereUtils;
import freemarker.ext.dom.NodeModel;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import java.io.*;
import java.net.URL;
import java.util.Map;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.base.Strings.emptyToNull;
import static com.google.common.collect.Maps.newHashMap;

@SuppressWarnings("serial")
public abstract class GenericBaseStep implements DeploymentStep {

    public static final String FREEMARKER_FILE_EXT = ".ftl";

    private int order;
    private String description;
    private HostContainer container;

    private String remoteWorkingDirPath;
    private boolean retainRemoteWorkingDirOnCompletion;

    private transient OverthereConnection localConn;
    private transient OverthereConnection remoteConn;
    private transient DeploymentExecutionContext ctx;
    private transient OverthereFile remoteWorkingDir;

    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    protected GenericBaseStep(int order, String description, HostContainer container) {
        this.order = order;
        this.description = description;
        this.container = container;
    }

    protected GenericBaseStep(int order, HostContainer container) {
        this(order, null, container);
    }

    @Override
    public int getOrder() {
        return order;
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public Result execute(DeploymentExecutionContext ctx) throws Exception {
        try {
            this.ctx = ctx;
            return doExecute();
        } finally {
            disconnect();
        }
    }

    protected abstract Result doExecute() throws Exception;

    protected DeploymentExecutionContext getCtx() {
        return ctx;
    }

    protected HostContainer getContainer() {
        return container;
    }

    public OverthereFile getRemoteWorkingDirectory() {
        if (remoteWorkingDir == null) {
            OverthereFile tempDir;
            if (Strings.isNullOrEmpty(getRemoteWorkingDirPath())) {
                tempDir = getRemoteConnection().getTempFile("generic_plugin", ".tmp");
            } else {
                tempDir = getRemoteConnection().getFile(getRemoteWorkingDirPath());
            }
            tempDir.mkdir();
            remoteWorkingDir = tempDir;
        }
        return remoteWorkingDir;
    }


    public OverthereConnection getLocalConnection() {
        if (localConn == null) {
            localConn = LocalConnection.getLocalConnection();
        }
        return localConn;
    }

    public OverthereConnection getRemoteConnection() {
        if (remoteConn == null) {
            remoteConn = container.getHost().getConnection();
        }
        return remoteConn;
    }

    protected void disconnect() {
        if (localConn != null)
            localConn.close();

        if (!Strings.isNullOrEmpty(getRemoteWorkingDirPath()) && !isRetainRemoteWorkingDirOnCompletion()) {
            getRemoteWorkingDirectory().deleteRecursively();
        }

        if (remoteConn != null)
            remoteConn.close();

        remoteWorkingDir = null;
        localConn = null;
        remoteConn = null;
    }

    public String getRemoteWorkingDirPath() {
        return remoteWorkingDirPath;
    }

    public void setRemoteWorkingDirPath(String remoteWorkingDirPath) {
        this.remoteWorkingDirPath = remoteWorkingDirPath;
    }

    public boolean isRetainRemoteWorkingDirOnCompletion() {
        return retainRemoteWorkingDirOnCompletion;
    }

    public void setRetainRemoteWorkingDirOnCompletion(boolean deleteWorkingDirOnCompletion) {
        this.retainRemoteWorkingDirOnCompletion = deleteWorkingDirOnCompletion;
    }

    public OverthereFile uploadResourceToWorkingDirectory(String resource) {
        final String fileName = substringAfterLast(resource, '/', resource);
        return uploadToWorkingDirectory(Resources.getResource(resource), fileName);
    }


    public OverthereFile uploadToWorkingDirectory(String content, String fileName) {
        OverthereFile target = getRemoteWorkingDirectory().getFile(fileName);
        OverthereUtils.write(content.getBytes(), target);
        return target;
    }

    public OverthereFile uploadToWorkingDirectory(File content, String fileName) {
        OverthereFile target = getRemoteWorkingDirectory().getFile(fileName);
        LocalFile.valueOf(content).copyTo(target);
        return target;
    }

    public OverthereFile uploadToWorkingDirectory(URL content, String fileName) {
        OverthereFile target = getRemoteWorkingDirectory().getFile(fileName);
        OutputStream out = target.getOutputStream();
        try {
            Resources.copy(content, out);
        } catch (IOException e) {
            throw new RuntimeIOException(e);
        } finally {
        	Closeables.closeQuietly(out);
        }
        return target;
    }

    public boolean hostFileExists(String remoteFile) {
        checkNotNull(emptyToNull(remoteFile));
        OverthereFile file = getRemoteConnection().getFile(remoteFile);
        return file.exists();
    }

    public String getHostLineSeparator() {
        return getContainer().getHost().getOs().getLineSeparator();
    }

    public String getHostFileSeparator() {
        return getContainer().getHost().getOs().getFileSeparator();
    }

    public String readHostFile(final String remoteFile) {
        checkNotNull(emptyToNull(remoteFile));
        OverthereFile file = getRemoteConnection().getFile(remoteFile);
        checkArgument(file.exists(), "File %s does not exist on host %s", remoteFile, getContainer().getHost());
        try {
        	InputStream in = file.getInputStream();
        	try {
        		byte[] bytes = ByteStreams.toByteArray(in);
        		return new String(bytes);
        	} finally {
        		in.close();
        	}
        } catch (IOException e) {
            throw new RuntimeIOException("Failed to read file " + remoteFile, e);
        }
    }

    public String[] readHostFileLines(final String remoteFile) {
        String data = readHostFile(remoteFile);
        Iterable<String> iterable = Splitter.on(getHostLineSeparator()).split(data);
        return Iterables.toArray(iterable, String.class);
    }

    public void createOrReplaceHostFile(String content, String remoteFile) {
        checkNotNull(emptyToNull(remoteFile));
        checkNotNull(content);
        OverthereFile file = getRemoteConnection().getFile(remoteFile);
        if (file.exists()) {
            logger.debug("File " + remoteFile + " already exists. Will delete before attempting to write.");
            file.delete();
        }

	    final ByteArrayInputStream from = new ByteArrayInputStream(content.getBytes());
	    final OutputStream to = file.getOutputStream();
	    try {
		    ByteStreams.copy(from, to);
        } catch (IOException e) {
            throw new RuntimeIOException("Failed to write to " + remoteFile, e);
        }
	    finally {
		    Closeables.closeQuietly(from);
		    Closeables.closeQuietly(to);
	    }
    }

    public NodeModel readHostXmlFileAsModel(final String remoteXmlFile) {
        checkNotNull(emptyToNull(remoteXmlFile));
        OverthereFile file = getRemoteConnection().getFile(remoteXmlFile);
        checkArgument(file.exists(), "File %s does not exist on host %s", remoteXmlFile, getContainer().getHost());
        InputStream remoteXmlStream = file.getInputStream();
        try {
            return NodeModel.parse(new InputSource(remoteXmlStream), false, true);
        } catch (ParserConfigurationException exc) {
            throw new RuntimeException("Cannot read xml file " + remoteXmlFile, exc);
        } catch (SAXException exc) {
            throw new RuntimeException("Cannot read xml file " + remoteXmlFile, exc);
        } catch (IOException exc) {
            throw new RuntimeIOException("Cannot read xml file " + remoteXmlFile, exc);
        } finally {
            Closeables.closeQuietly(remoteXmlStream);
        }
    }

    public String evaluateTemplate(String templatePath, Map<String, Object> vars) {
        Configuration cfg = ConfigurationHolder.getConfiguration();
        try {
            Template template = cfg.getTemplate(templatePath);
            StringWriter sw = new StringWriter();
            template.createProcessingEnvironment(vars, sw, new CiAwareObjectWrapper(new WorkingFolderUploader())).process();
            return sw.toString();
        } catch (IOException e) {
            throw new RuntimeIOException(e);
        } catch (TemplateException e) {
            throw new RuntimeException(e);
        }
    }

    public String resolveOsSpecificTemplate(String template) {
        if (classpathResourceExists(template + FREEMARKER_FILE_EXT)) {
            return template + FREEMARKER_FILE_EXT;
        }

        String osSpecificTemplate = template;

        String scriptExt = substringAfterLast(osSpecificTemplate, '.');
        if (scriptExt == null) {
            OperatingSystemFamily os = getContainer().getHost().getOs();
            osSpecificTemplate = osSpecificTemplate + os.getScriptExtension();
        }

        if (!classpathResourceExists(osSpecificTemplate)) {
            String osSpecificScriptTemplate = osSpecificTemplate + FREEMARKER_FILE_EXT;
            if (!classpathResourceExists(osSpecificScriptTemplate))
                throw new IllegalArgumentException("Resource " + osSpecificTemplate + " not found in classpath");
            else
                osSpecificTemplate = osSpecificScriptTemplate;
        }

        return osSpecificTemplate;
    }

    public boolean classpathResourceExists(String resource) {
        return Thread.currentThread().getContextClassLoader().getResource(resource) != null;
    }

    protected void evaluateTemplate(OverthereFile renderTo, String templatePath, Map<String, Object> vars) {
        Configuration cfg = ConfigurationHolder.getConfiguration();
        OutputStream out = renderTo.getOutputStream();
        try {
            Template template = cfg.getTemplate(templatePath);
            template.process(vars, new OutputStreamWriter(out));

            if (logger.isDebugEnabled()) {
                InputStream in = renderTo.getInputStream();
                try {
                    logger.debug(new String(ByteStreams.toByteArray(in)));
                } finally {
                    Closeables.closeQuietly(in);
                }
            }

        } catch (IOException e) {
            throw new RuntimeIOException(e);
        } catch (TemplateException e) {
            throw new RuntimeException(e);
        } finally {
            Closeables.closeQuietly(out);
        }
    }

    public String substringAfterLast(String str, char sub, String defaultValue) {
        String s = substringAfterLast(str, sub);
        if (s == null) {
            return defaultValue;
        }
        return s;
    }

    public String substringAfterLast(String str, char sub) {
        int pos = str.lastIndexOf(sub);
        if (pos == -1) {
            return null;
        } else {
            return str.substring(pos + 1);
        }
    }

    private class WorkingFolderUploader implements FileUploader {
        private Map<String, String> uploadedFiles = newHashMap();

        @Override
        public String upload(OverthereFile file) {
            if (uploadedFiles.containsKey(file.getName())) {
                return uploadedFiles.get(file.getName());
            }
            OverthereFile uploadedFile = getRemoteWorkingDirectory().getFile(file.getName());
            if (!uploadedFile.exists()) {
                file.copyTo(uploadedFile);
            }
            uploadedFiles.put(file.getName(), uploadedFile.getPath());
            return uploadedFile.getPath();
        }
    }
}
